###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


import tacticenv
import sys, re, getopt, os

from pyasm.security import Batch
from pyasm.biz import Project
from pyasm.command import Command
from pyasm.search import Search, DbContainer, SObject, Sql
from pyasm.search.upgrade import Upgrade
from pyasm.common import Container, Environment



# This has been moved to pyasm.search
"""
class Upgrade(object):

    def __init__(my, version, is_forced=False, project_code=None, quiet=False):
        my.to_version = version
        my.is_forced = is_forced
        #my.is_confirmed = is_confirmed

        my.project_code = project_code
        my.quiet = quiet

        
    def execute(my):
        error_list = []
        sthpw_search = Search("sthpw/project")
        sthpw_search.add_filter('code','sthpw')
        sthpw_search.set_show_retired(True)
        sthpw_proj = sthpw_search.get_sobject()

        search = Search("sthpw/project")
        if my.project_code:
            search.add_filter("code", my.project_code)
        else:
            search.add_enum_order_by("type", ['sthpw','prod','flash','game','design','simple', 'unittest'])
        projects = search.get_sobjects()

        project_codes = SObject.get_values(projects, 'code')
        
        # append sthpw project in case it's retired
        if 'sthpw' not in project_codes and sthpw_proj:
            if not my.project_code:
                projects.insert(0, sthpw_proj)
            sthpw_proj.reactivate()



        current_dir = os.getcwd()
        output_file = '%s/upgrade_output.txt' % current_dir
        if os.path.exists(output_file):
            os.unlink(output_file)
        ofile = open(output_file, 'a')
        import datetime
        ofile.write('Upgrade Time: %s\n\n' %datetime.datetime.now())



        # dynamically generate
        #sql = DbContainer.get(code)
        database_type = Sql.get_default_database_type()
        if database_type in ['Sqlite', 'MySQL']:
            # general an upgrade
            import imp
            from pyasm.search.upgrade.sqlite import *
            from pyasm.search.upgrade.mysql import *

            namespaces = ['default', 'simple', 'sthpw', 'config']
            for namespace in namespaces:
                if database_type == 'Sqlite':
                    file_path = convert_sqlite_upgrade(namespace)
                else:
                    file_path = convert_mysql_upgrade(namespace)

                (path, name) = os.path.split(file_path)
                (name, ext) = os.path.splitext(name)
                (file, filename, data) = imp.find_module(name, [path])
                module = imp.load_module(name, file, filename, data)

                class_name = "%s%sUpgrade" % (database_type,namespace.capitalize())
                exec("%s = module.%s" % (class_name, class_name) )


        # load all the default modules
        from pyasm.search.upgrade.project import *

        for project in projects:
            
            code = project.get_code()
            if code == "sthpw":
                type = "sthpw"
            else:
                type = project.get_type()

            if not type:
                type = 'default'


            if not my.quiet:
                print project.get_code(), type
                print "-"*30

            # if the project is admin, the just ignore for now
            if code == 'admin':
                continue
            
            if not project.database_exists():
                ofile.write("*" * 80 + '\n')
                msg =  "Project [%s] does not have a database\n"% project.get_code()
                ofile.write(msg)
                print msg
                ofile.write("*" * 80 + '\n\n')
                continue


            upgrade = None

            if database_type == 'Sqlite':
                upgrade_class = "Sqlite%sUpgrade" % type.capitalize()
                conf_upgrade = eval("SqliteConfigUpgrade()")
            else:
                upgrade_class = "%sUpgrade" % type.capitalize()
                conf_upgrade = eval("ConfigUpgrade()")
            upgrade = eval("%s()" % upgrade_class)

            # upgrade config (done for every project but sthpw)
            conf_upgrade.set_project(project.get_code())
            conf_upgrade.set_to_version(my.to_version)
            conf_upgrade.set_forced(my.is_forced)
            conf_upgrade.set_quiet(my.quiet)
            conf_upgrade.execute()
            
            # append the errors for each upgrade
            key = '%s|%s' %(project.get_code(), conf_upgrade.__class__.__name__)
            error_list.append((conf_upgrade.__class__.__name__, project.get_code(), \
                Container.get_seq(key)))


            # perform the upgrade to the other tables
            if upgrade:
                upgrade.set_project(project.get_code() )
                upgrade.set_to_version(my.to_version)
                upgrade.set_forced(my.is_forced)
                upgrade.set_quiet(my.quiet)
                #Command.execute_cmd(upgrade)
                # put each upgrade function in its own transaction
                # carried out in BaseUpgrade
                upgrade.execute()
                
                # append the errors for each upgrade
                key = '%s|%s' %(project.get_code(), upgrade.__class__.__name__)
                error_list.append((upgrade.__class__.__name__, project.get_code(), \
                    Container.get_seq(key)))



            
            import datetime
            project.set_value("last_db_update", datetime.datetime.now())
            
            if project.has_value('last_version_update'):
                last_version = project.get_value('last_version_update')
                if my.to_version > last_version:
                    project.set_value("last_version_update", my.to_version)
            else: 
                # it should be getting the upgrade now, redo the search
                print "Please run upgrade_db.py again, the sthpw db has just been updated"
                return
            project.commit(triggers=False)

        # print the errors for each upgrade
        
        
        
        for cls_name, project_code, errors in error_list:
            print
            if not my.quiet:
                print "Errors for %s [%s]:" %(project_code, cls_name)
            ofile.write("Errors for %s [%s]:\n" %(project_code, cls_name))
            if not my.quiet:
                print "*" * 80
            ofile.write("*" * 80 + '\n')
            for func, error in errors:
                if not my.quiet:
                    print '[%s]' % func
                    print "-" * 70
                    print error
                ofile.write('[%s]\n' % func)
                ofile.write("-" * 70 + '\n')
                ofile.write('%s\n' %error)
        file.close()

        if my.quiet:
            print "Please refer to the upgrade_output.txt file for any upgrade messages."
            print
        # handle sthpw database separately.  This ensures that the project entry
        # gets created if none exists.
        #print "sthpw"
        #print "-"*30
        #upgrade = SthpwUpgrade()
        #upgrade.set_project("sthpw")
        #Command.execute_cmd(upgrade)

"""

if __name__ == '__main__':
    is_forced = False
    is_confirmed = False
    tacticversion = "2.5.0"
    project_code = None
    quiet = False


    try:
        opts, args = getopt.getopt(sys.argv[1:], "fyhp:q", ["force","yes","help","project=","quiet"])
    except getopt.error, msg:
        print msg
        sys.exit(2)
    # process options
    for o, a in opts:
	if o in ("-y", "--yes"):
            is_confirmed = True
        if o in ("-f", "--force"):
            is_forced = True
        if o in ("-h", "--help"):
            print "python upgrade_db.py [-fy] [%s]" % tacticversion
            sys.exit(0)
        if o in ("-p", "--project"):
            project_code = a
        if o in ("-q", "--quiet"):
            quiet = True
   

    if len(args) == 0:
        # trace the version of TACTIC that houses this upgrade_db.py script
        dir = os.getcwd()
        file_path = sys.modules[__name__].__file__
        full_path = os.path.join(dir, file_path)
        pat = re.compile(r'\\|/')
        path_parts = pat.split(full_path)
        index = path_parts.index('src')
        # used to be -3
        path_parts = path_parts[0:index]
        f = open('%s/VERSION' %'/'.join(path_parts), 'r')
        current_version =  f.readline().strip()
        msg = "Upgrading to version [%s]" % current_version
        if is_forced:
            msg = "Upgrading to version [%s] with force option" % current_version
        if not is_confirmed:
	    answer = raw_input(" %s. Continue (y/n): " %msg)
            if answer == "y":
                pass
            elif answer == 'n':
                sys.exit(0)
            else:
                print "Only y or n is accepted. Exiting..."
                sys.exit(0)
        version = current_version
    elif len(args) == 1:
        version = args[0]
    else:
        print "Either 1 or no argument can be accepted."

    if version != Environment.get_release_version():
        
        answer = raw_input("[%s] does not match currently running TACTIC version [%s]. If it is a lower version number, it will probably not work. -f must be used to continue. For regular upgrade, you have not symlinked to the new TACTIC version. Continue (y/n): " \
                %(version, Environment.get_release_version()))
        if answer == "y":
            if not is_forced:
                print "-f option must be used in this situation. "
                sys.exit(0)
            pass
        elif answer == 'n':
            sys.exit(0)
        else:
            print "Only y or n is accepted. Exiting..."
            sys.exit(0)

    # check if some projects are already in newer version
    Batch()
    search = Search("sthpw/project")
    if project_code:
        search.add_filter("code", project_code)
    else:
        search.add_enum_order_by("type", ['sthpw','prod','flash','game','design','simple', 'unittest'])
    projects = search.get_sobjects()
    project_dict = {}
    for project in projects:
        last_version = project.get_value('last_version_update', no_exception=True)
        if last_version > version:
            project_dict[project.get_code()] = last_version

    if project_dict:
        data = []
        for key, value in project_dict.items():
            data.append('    %s --- %s' %(key, value))
        answer = raw_input("Several projects are already in newer versions:\n%s\n"\
                "These will not be upgraded unless their last_version_update is lowered. "\
                "Continue (y/n): " %('\n'.join(data)))
                
        if answer == "y":
            pass
        elif answer == 'n':
            sys.exit(0)
        else:
            print "Only y or n is accepted. Exiting..."
            sys.exit(0)

    p = re.compile(r'\d+.\d+.\d+(.\w+)?$')
    if not p.match(version):
        print 
        print "Version pattern is invalid. Examples for version are 2.0.0 or 2.0.0.rc02. If you are just upgrading to the current version, just run: "
        print
        print "python upgrade_db.py"
        sys.exit(0)

    version.replace('.', '_')
    
    upgrade = Upgrade(version, is_forced, project_code=project_code, quiet=quiet)
    upgrade.execute()

    print "Upgrade to version [%s] finished." % version
    dir = os.getcwd()
    output_file = '%s/upgrade_output.txt' %dir
    print "Upgrade output file saved in [%s]" %output_file
    



