###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ["DateChartWdg"]

from pyasm.common import Environment, Common
from pyasm.biz import Project
from pyasm.web import Widget, DivWdg, HtmlElement, WebContainer
from pyasm.widget import SelectWdg, TextWdg
from pyasm.search import Search, SearchType
from tactic.ui.common import BaseRefreshWdg

from datetime import datetime
from dateutil import rrule

class DateChartWdg(BaseRefreshWdg):


    def to_utc(my, date):
        diff = date - datetime(1970, 1, 1)
        seconds = diff.days*86400 + diff.seconds
        return seconds


    def get_sobjects(my):

        # x-axis = sobject
        # y-axis = values

        # now how to generate those sobjects
        year = 2010
        start_date = datetime(year, 1, 1)
        end_date = datetime(year+1, 1, 1)
        dates = list(rrule.rrule(rrule.MONTHLY, bymonthday=1, dtstart=start_date, until=end_date))

        # try to add some values here

        # number of checkins per week?
        # create an sobject for each date
        # FIXME: use dummy search_type
        sobjects = []
        prev_date = None
        for date in dates:
            if not prev_date:
                prev_date = date
                continue

            # have to do this wonky date stuff until I fix the bug with spaces
            # in literals in filters
            start_date = str(prev_date).split(" ")[0]
            end_date = str(date).split(" ")[0]
            #expr = "@COUNT(sthpw/snapshot['timestamp','>=','%s']['timestamp','<','%s'])" % ( start_date, end_date )
            #expr = "@SUM( @GET(sthpw/file['timestamp','>=','%s']['timestamp','<','%s'].st_size) )" % ( start_date, end_date )
            expr = "@COUNT( sthpw/snapshot['timestamp','>=','%s']['timestamp','<','%s'] )" % ( start_date, end_date )
            value = Search.eval(expr)


            #sobject = SearchType.create("sthpw/chart")
            sobject = SearchType.create("sthpw/login")

            sobject.set_value("x", my.to_utc(date))
            sobject.set_value("value", value )
            sobjects.append(sobject)

            prev_date = date


        # this gets all the sobjects
        sobject_expr = "@SOBJECT(sthpw/snapshot)"
        x_col = "timestamp"
        value_col = '@AVG(.completion)'



        # this gets all the sobjects
        sobject_expr = "@SOBJECT(sthpw/snapshot)"
        x_col = "timestamp"
        value_col = '@GET(.completion)'


        return sobjects


    def get_display(my):

        sobjects = my.get_sobjects()
        chart_wdg = my.get_chart_wdg(sobjects)

        top = DivWdg()
        top.add(chart_wdg)
        return top
 

    def get_chart_wdg(my, sobjects):
        element = ChartElement("line")

        x_min = Search.eval("@MIN(.x)", sobjects)
        x_max = Search.eval("@MAX(.x)", sobjects)

        y_min = Search.eval("@MIN(.value)", sobjects)
        y_min = 0
        y_max = Search.eval("@MAX(.value)", sobjects)
        y_steps = int( float(y_max - y_min) / 10 )


        #values = [
        #    1,2,3,4,5
        #]

        # convert sobjects to values in the chart
        values = []
        for sobject in sobjects:
            value = {
                'x': sobject.get_value("x"),
                'y': sobject.get_value("value")
            }
            values.append(value)
            
            
        chart_data = ChartData()

        x_axis = {
            "min": x_min,
            "max": x_max,
            "steps": 86400*7,
            "labels":{
              "rotate":345,
              "steps":86400*30,
              "visible-steps":1,
              "text":"#date: M jS, Y#"
            },
            'grid-colour': '#555555',
        }
        chart_data.set_value('x_axis', x_axis)

        y_axis = {
            "min": y_min,
            "max": y_max,
            "steps": y_steps,
            "labels":{
              "steps": 2,
              "visible-steps":2,
            },
            'grid-colour': '#555555',
        }
        chart_data.set_value('y_axis', y_axis)




        element.set_values(values)

        chart_data.add_element(element)

        chart = ChartWdg(chart=chart_data)

        return chart
       

