###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#

__all__ = [
        'BaseFilterElementWdg',
        'SelectFilterElementWdg',
        'KeywordFilterElementWdg',
        'DateFilterElementWdg',
        'DateRangeFilterElementWdg',
        'LoginFilterElementWdg',
        'MultiSelectFilterElementWdg',
        'MultiFieldFilterElementWdg',
        'CompoundValueFilterElementWdg',
        'ExpressionFilterElementWdg',
        'ReplaceWithValueExpressionFilterElementWdg'
]

from pyasm.common import Common
from pyasm.web import DivWdg, SpanWdg, Table, WebContainer
from pyasm.widget import CheckboxWdg, SelectWdg, TextWdg, HiddenWdg
from pyasm.search import Search, SearchException

from tactic.ui.common import BaseRefreshWdg

from filter_data import FilterData


class BaseFilterElementWdg(BaseRefreshWdg):
    '''represents the base filter'''
    def __init__(my, **kwargs):
        my.values = {}
        my.show_title = False
        my.set_flag = False
        super(BaseFilterElementWdg, my).__init__(**kwargs)

    def is_visible(my):
        return True

    def set_show_title(my, flag):
        my.show_title = flag


    def is_set(my):
        '''indicates whether this filter has values set that will
        oontribute to the search'''
        return my.set_flag


    def set_values(my, values):
        my.values = values

    def alter_search(my, search):
        pass

    def get_display(my):
        pass



class SelectFilterElementWdg(BaseFilterElementWdg):


    def is_set(my):
        value = my.values.get("value")
        if value:
            return True
        else:
            if not my.values and my.kwargs.get('default'):
                return True
            else:
                return False
        
        

    def alter_search(my, search):

        expression = my.kwargs.get("column")
        if not expression:
            return
        parts = expression.split(".")
        search_types = parts[:-1]
        column = parts[-1]

        value = my.values.get("value")
        #print "value: ", value
        if not value:
            default =  my.kwargs.get('default')
            if not my.values and default:
                value = default
            else:
                return

        # op should come from my.values
        op = my.values.get("op")
        if not op:
            op = '='

        # go through the hierarchy
        search2 = None
        sobjects = None
        if len(search_types) > 1:
            search_types.reverse()
            for search_type in search_types:
                if sobjects == None:
                    search2 = Search(search_type)
                    search2.add_filter(column, value, "=")
                    sobjects = search2.get_sobjects()
                else:
                    search2 = Search(search_type)
                    search2.add_relationship_filters(sobjects)
                    sobjects = search2.get_sobjects()

                if not sobjects:
                    # if the trail ends, then set the null filter and exit
                    search.set_null_filter()
                    return


        elif not search_types:
            if op == '!=':
                search.add_op('begin')
                search.add_filter(column, value, op)
                search.add_filter(column, None)
                search.add_op('or')
            elif op in ['~']:
                filters = [[column,'~',value]]
                search.add_op_filters(filters)
            else:
                search.add_filter(column, value, op)
            return
        else:
            search_type = search_types[0]

            # get all the sobjects at the appropriate hierarchy
            try:
                search2 = Search(search_type)
                if value:
                    if op == 'exists':
                        search.add_relationship_search_filter(search2, op=value)
                        return

                    else:
                        #search2.add_filter(column, value, op)
                        if op == '!=':
                            search2.add_op('begin')
                            search2.add_filter(column, value, op)
                            search2.add_filter(column, None)
                            search2.add_op('or')
                        elif op in ['~']:
                            filters = [[column,'~',value]]
                            search2.add_op_filters(filters)
                        else:
                            search2.add_filter(column, value, op)
                search.add_relationship_search_filter(search2, op="in")
            except SearchException, e:
                raise SearchException('[%s] in simple search definition may have syntax error. %s ' %(expression, e.__str__()))
            return

        if sobjects:
            search.add_relationship_filters(sobjects)
        else:
            search.set_null_filter()



    def get_display(my):

        div = DivWdg()
        div.add_style("width: 350px")

        select = SelectWdg("value")

        # TODO: this is needed for multiple selection, but it is ugly
        #select.set_attr("multiple", "1")
        #select.add_attr("spt_is_multiple", "true")
        # if there is a link search already, don't use default
        if my.values and my.kwargs.get('default'):
            my.kwargs.pop('default')
        select.set_options(my.kwargs)
        select.add_empty_option("-- Select --")

        select.add_behavior( {
            'type': 'change',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_filter_top");
            var set_icon = top.getElement(".spt_filter_set");
            if (!set_icon) return;

            if (bvr.src_el.value == '') {
                set_icon.setStyle("display", "none");
            }
            else {
                set_icon.setStyle("display", "");
            }
            '''
        } )

        # this is needed so they don't cross contaminate
        # FIXME: this is probably a bug in SelectWdg
        select.set_value('')
        value = my.values.get("value")
        if value:
            select.set_value(value)


        if my.show_title:
            title_div = DivWdg()
            div.add(title_div)
            title_div.add_style("float: left")

            title = my.get_name()
            title = Common.get_display_title(title)
            title_div.add("%s:" % title )
            title_div.add_style("width: 80px")
            title_div.add_style("font-weight: bold")
            title_div.add_style("margin-left: 15px")
            title_div.add_style("padding-top: 2px")

        op = my.kwargs.get("op")
        if op == 'exists':
            div.add("&nbsp;&nbsp;&nbsp;is&nbsp;&nbsp;&nbsp;")
        elif op == '~':
            div.add("&nbsp;&nbsp;&nbsp;contains&nbsp;&nbsp;&nbsp;")
        else:
            op_select = SelectWdg("op")
            op_select.set_option("labels", "is|is not|contains")
            op_select.set_option("values", "=|!=|~")

            value = my.values.get("op")
            if value:
                op_select.set_value(value)

            div.add(op_select)

        select.add_style("width: 150px")
        div.add(select)
        return div



class KeywordFilterElementWdg(BaseFilterElementWdg):


    def init(my):

        # FIXME: this is dependent on the default database and not
        # on the database that may actually be searched on
        from pyasm.search import Sql
        database_type = Sql.get_default_database_type()

        has_index = False
        if database_type == 'PostgreSQL':

            database_version = Sql.get_default_database_version()
            major = database_version[0]
            minor = database_version[1]

            if major >= 9:
                has_index = True
            elif major < 8:
                has_index = False
            else:
                if minor >= 4:
                    has_index = True
                else:
                    has_index = False

        my.has_index = has_index



    def alter_search(my, search):
        value = my.values.get("value")
        if not value:
            return

        name = my.get_name()

        partial = my.values.get("partial")

        try:
            value.encode('ascii')
        except UnicodeEncodeError:
            is_ascii = False
        else:
            is_ascii = True

        """
        search_type = "sthpw/sobject_list"
        column = "keywords"
        search2 = Search(search_type)
        search2.add_text_search_filter(column, keywords)
        sobjects = search2.get_sobjects()
        """

        if my.has_index and is_ascii:
            value = value.replace(",", " ")
            keywords = value.split(" ")
            name = my.get_name()
            search.add_text_search_filter(name, keywords)

            if partial:
                search.add_keyword_filter(name, keywords)
                search.add_op("or")

        else:
            value = value.replace(",", " ")
            keywords = value.split(" ")


            search.add_keyword_filter(name, keywords)


    def get_display(my):

        div = DivWdg()
        div.add_style("width: 300px")

        if my.show_title:
            title = my.get_name()
            title = Common.get_display_title(title)

            #title_div = DivWdg()
            #div.add(title_div)
            #title_div.add("%s: " % title )
            #title_div.add_style("float: left")
            div.add("<b>%s: &nbsp;</b>" % title )


        text = TextWdg("value")
        div.add(text)
        text.add_style("-moz-border-radius: 4px")
        text.add_style("padding: 2px")

        if my.has_index:
            #from tactic.ui.input.text_input_wdg import TextInputWdg
            #text = TextInputWdg(name="value")
            #div.add(text)
            #div.add("<br/>")
            checkbox = CheckboxWdg("partial")
            checkbox.add_attr("title", "Use partial word match (slower)")
            div.add(checkbox)
            #div.add("partial word match (slow)")
        else:
            hidden = HiddenWdg("partial")
            div.add(hidden)
            hidden.set_value("true")



        return div



class DateFilterElementWdg(BaseFilterElementWdg):
    '''This filter uses a subselect so that it can look for dates that
    are on another table than the main serach'''
    def alter_search(my, search):

        expression = my.kwargs.get("column")
        if not expression:
            return

        search_type, date_col = expression.split(".")

        start_date = my.values.get("start_date")
        end_date = my.values.get("end_date")

        from pyasm.search import Search
        search2 = Search(search_type)
        search2.add_date_range_filter(date_col, start_date, end_date)

        #sobjects = search2.get_sobjects()
        search.add_relationship_search_filter(search2)


    def get_display(my):

        span = DivWdg()
        span.add_style("float: left")
        #div.add_style("border: solid blue 1px")
        op = DivWdg("is between&nbsp;&nbsp;&nbsp;")
        op.add_style("float: left")
        span.add(op)

        from tactic.ui.widget import CalendarInputWdg
        cal1 = CalendarInputWdg("start_date")
        cal1.add_style("float: left")
        span.add(cal1)

        spacing = DivWdg("&nbsp;and&nbsp;")
        spacing.add_style("float: left")
        span.add(spacing)

        cal2 = CalendarInputWdg("end_date")
        cal2.add_style("float: left")
        span.add(cal2)
        return span




class DateRangeFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        start_col = my.kwargs.get("start_date_col")
        end_col = my.kwargs.get("end_date_col")



        name = my.get_name()
        if not start_col:
            start_col = "%s_start_date" % name
        if not end_col:
            end_col = "%s_end_date" % name

        # find a relative search type if needed
        parts = start_col.split(".")
        search_types = parts[:-1]
        start_col = parts[-1]

        # find a relative search type if needed
        parts = end_col.split(".")
        search_types2 = parts[:-1]
        end_col = parts[-1]

        if search_types != search_types2:
            raise SearchException('Search types for start and end column must match')


        # just take the first one
        if search_types:
            search_type = search_types[0]
        else:
            search_type = search.get_search_type()


        start_date = my.values.get("start_date")
        end_date = my.values.get("end_date")

        if not start_date and not end_date:
            return


        operator = my.get_option("op")
        if operator != 'not in':
            operator = 'in'

        from pyasm.search import Search
        search2 = Search(search_type)
        search2.add_dates_overlap_filter(start_col, end_col, start_date, end_date, op="in")
        search.add_relationship_search_filter(search2, op=operator)

        return

        # Add in the ability to do something like ...
        #select * from sequence where code not in (select distinct sequence_code from shot);
        relationship = None
        if relationship == 'search_type':
            where = '''id not in (select distinct search_id from task where search_type like 'prod/asset?project=%')'''
        else:
            where = '''id not in (select distinct sequence_code from shot)'''

        #search.add_empty_related_filter("sthpw/task", op='not in')





    def get_display(my):

        div = DivWdg()

        name = my.get_name()

        title = Common.get_display_title(name)

        operator = my.get_option("op")
        if operator != 'not in':
            operator = 'in'

        table = Table()
        div.add(table)
        table.add_row()
        table.add_color("color", "color")

        name_div = DivWdg()
        name_div.add("%s " % title)
        table.add_cell(name_div)

        #div.add_style("border: solid blue 1px")
        if operator == 'in':
            op = DivWdg("overlaps ")
        else:
            op = DivWdg("does not overlap ")
        table.add_cell(op)

        from tactic.ui.widget import CalendarInputWdg
        cal1 = CalendarInputWdg("start_date")
        table.add_cell(cal1)

        start_date = my.values.get("start_date")
        if start_date:
            cal1.set_value(start_date)
 
        table.add_cell(" and ")

        cal2 = CalendarInputWdg("end_date")
        table.add_cell(cal2)

        end_date = my.values.get("end_date")
        if end_date:
            cal2.set_value(end_date)

        return div



class ExpressionFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        prefix = my.values.get("prefix")
        #column = my.values.get("%s_column" % prefix)

        option = my.values.get("option")
        if option == 'my_items':
            expr = my.get_option("expression")
            column = my.get_option("column")

            from pyasm.biz import ExpressionParser
            parser = ExpressionParser()
            results = parser.eval(expr)
            if results:
                search.add_filters(column, results)
            else:
                search.add_filter(column, 'NULL')

        elif option == 'all_items':
            pass
       

    def get_display(my):

        # FIXME
        title = my.get_option("title")

        div = SpanWdg()
        div.add("%s: " % title)
        checkbox = CheckboxWdg("option")
        checkbox.set_attr("value", "my_items")
        checkbox.set_checked()
        div.add(checkbox)
        div.add("&nbsp;&nbsp;")
        div.add("All")
        checkbox = CheckboxWdg("option")
        checkbox.set_attr("value", "all_items")
        div.add(checkbox)
        return div



# --------------------------------------------------------------------------------------------------------------------
# ReplaceWithValueExpressionFilterElementWdg ...
#
#     This is really just the ExpressionFilterElementWdg but with the ability to have a text field entry
#     for a value to replace within your expression. Where-ever you place $REPLACE in your expression string
#     will be replaced with the value you enter in the text field entry. Below is some example configuration
#     XML for this widget ...
#
#     <element name='has_subtask_with_lead'> 
#       <display class='tactic.ui.filter.ReplaceWithValueExpressionFilterElementWdg'>
#         <expression>@GET(MMS/subtask['lead_name','$REPLACE'].job_id)</expression>
#         <column>id</column>
#         <field_size>22</field_size>
#         <field_label>matching:</field_label>
#       </display>
#     </element>
#
class ReplaceWithValueExpressionFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        expr = my.get_option("expression")
        field_value = my.values.get("replace_value")

        expr = expr.replace("$REPLACE", field_value)

        column = my.get_option("column")

        from pyasm.biz import ExpressionParser
        parser = ExpressionParser()
        results = parser.eval(expr)
        if results:
            search.add_filters(column, results)


    def get_display(my):

        field_size = my.get_option("field_size")
        if not field_size:
            field_size = "32"
        else:
            field_size = "%s" % field_size

        field_label = my.get_option("field_label")
        if not field_label:
            field_label = ""

        div = SpanWdg()
        div.add(" %s " % field_label)
        text = TextWdg("replace_value")
        text.add_attr("size", field_size)
        div.add(text)
        return div



class LoginFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        prefix = my.values.get("prefix")
        column = my.values.get("%s_column" % prefix)

        option = my.values.get("option")
        if option == 'my_items':
            search.add_user_filter()
        elif option == 'all_items':
            pass
       

    def get_display(my):

        # FIXME
        title = "Jobs"

        div = SpanWdg()
        div.add("My %s: " % title)
        checkbox = CheckboxWdg("option")
        checkbox.set_attr("value", "my_items")
        checkbox.set_checked()
        div.add(checkbox)
        div.add("&nbsp;&nbsp;")
        div.add("All User %s: " % title)
        checkbox = CheckboxWdg("option")
        checkbox.set_attr("value", "all_items")
        div.add(checkbox)
        return div



#
# FIXME: hardcoded for MMS
#
class MultiSelectFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        filters = ['MMS/job.job_number_prefix', 'MMS/job.job_number_year', 'MMS/job.job_number']
        for filter in filters:
            value = my.values.get(filter)
            if value:
                #search.add_relationship_filter(filter)
                search_type, column = filter.split('.')
                search.add_filter(column, value)



    def get_display(my):

        top = SpanWdg()
        top.add("&nbsp; matches &nbsp;")

        filters = ['MMS/job.job_number_prefix', 'MMS/job.job_number_year', 'MMS/job.job_number']

        for filter in filters:
            text = TextWdg(filter)
            text.add_attr("size", "5")
            top.add(text)
            top.add("&nbsp;-&nbsp;")

        return top



# This is a generalized version of the MultiSelectFilterElementWdg
#
class MultiFieldFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        field_list_option = my.get_option("field_list")
        if not field_list_option:
            return

        stmt = 'field_info_list = %s' % field_list_option.replace("\n"," ")
        try:
            exec stmt
        except:
            return

        for field_info in field_info_list:

            field = field_info.get('field')
            op = field_info.get('op')
            if not op:
                op = '='

            if field:
                value = my.values.get(field)
                to = field_info.get('to')
                if to:
                    if to.endswith('()'):
                        stmt = 'value = value.%s' % to
                    else:
                        # assume casting to other value besides string ...
                        stmt = 'value = %s(value)' % to
                    try:
                        exec stmt
                    except:
                        # TODO ... proper error message here?
                        continue
                if value:
                    # print "field is [%s] ... op is [%s] ... to is [%s] ... value is [%s]" % (field, op, to, value)
                    search.add_filter(field, value, op)


    def set_configuration_error(my, top_el, error_message):

        top_el.add("Error in Configuration -- %s" % error_message)
        top_el.add_styles("color: orange; font-style: italic; font-weight: bold;")


    def get_display(my):

        top = SpanWdg()

        field_list_option = my.get_option("field_list")
        if not field_list_option:
            my.set_configuration_error( top, "No 'field_list' option provided" )
            return top

        top.add("&nbsp; matches &nbsp;")
        stmt = 'field_info_list = %s' % field_list_option.replace("\n"," ")
        try:
            exec stmt
        except:
            my.set_configuration_error( top, "badly formed 'field_list' option" )
            return top

        # field_list option should look like this:
        #     [ {'field': 'job_number_prefix', 'size': '1', 'maxlength': '1', 'label': 'Prefix', 'to': 'upper()' },
        #       {'field': 'job_number_year', 'size': '2', 'maxlength': '2', 'label': 'Year (2 digits)', 'to': 'int' },
        #       {'field': 'job_number', 'size': '5', 'maxlength': '5', 'label': 'Number', 'to': 'int' },
        #     ]
        #
        # ... you can also specify a specific op in each of the above ... defaults to 'op': '='

        default_size = 10

        done_first = False
        for field_info in field_info_list:
            field = field_info.get('field')
            if field:
                text = TextWdg(field)
                size = field_info.get('size')
                if not size:
                    size = default_size

                text.add_attr("size", "%s" % size)
                maxlength = field_info.get('maxlength')
                if maxlength:
                    text.add_attr("maxlength", "%s" % maxlength)

                label = field_info.get('label')
                if label:
                    if done_first:
                        top.add("&nbsp;&nbsp;&nbsp;%s: " % label)
                    else:
                        top.add("%s: " % label)
                top.add(text)

                done_first = True

        return top


# --------------------------------------------------------------------------------------------------------------------
# CompoundValueFilterElemenetWdg ...
#
#     This is used to set up multiple text field entries to build a single value up to compare to a specific
#     column ... example is for MMS job number. Below is some example configuration XML for this widget ...
#
# <display class='tactic.ui.filter.CompoundValueFilterElementWdg'>
#     <field_list> [
#         { 'field': 'job_number_prefix', 'label': 'Prefix', 'size': '1', 'maxlength': '1', 'to': 'upper()' },
#         { 'field': 'job_number_year', 'label': 'Year (2 digit)', 'size': '2', 'maxlength': '2', 'to': 'zfill(2)' },
#         { 'field': 'job_number', 'label': 'Number', 'size': '5', 'maxlength': '5', 'to': 'zfill(5)' }
#     ] </field_list>
#     <column>job_number_full</column>
#     <compound_value_expr>{job_number_prefix}{job_number_year}-{job_number}</compound_value_expr>
#     <op>=</op>
# </display> 
#
class CompoundValueFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        field_list_option = my.get_option("field_list")
        if not field_list_option:
            return

        stmt = 'field_info_list = %s' % field_list_option.replace("\n"," ")
        try:
            exec stmt
        except:
            return

        column = my.get_option("column")
        if not column:
            print
            print
            print "*** ERROR: no column specified for CompoundValueFilterElementWdg"
            print
            print
            return

        field_map = {}

        for field_info in field_info_list:

            field = field_info.get('field')
            if field:
                value = my.values.get(field)
                to = field_info.get('to')
                if to:
                    if to.endswith(')'):
                        stmt = 'value = value.%s' % to
                    else:
                        # assume casting to other value besides string ...
                        stmt = 'value = %s(value)' % to
                    try:
                        exec stmt
                    except:
                        # TODO ... proper error message here?
                        continue
                if value:
                    field_map[field] = value

        compound_value = my.get_option("compound_value_expr")
        if not compound_value:
            return

        for f,v in field_map.iteritems():
            compound_value = compound_value.replace( "{%s}" % f, v )

        op = my.get_option("op")
        if not op:
            op = "="

        search.add_filter( column, compound_value, op)


    def set_configuration_error(my, top_el, error_message):

        top_el.add("Error in Configuration -- %s" % error_message)
        top_el.add_styles("color: orange; font-style: italic; font-weight: bold;")


    def get_display(my):

        top = SpanWdg()

        field_list_option = my.get_option("field_list")
        if not field_list_option:
            my.set_configuration_error( top, "No 'field_list' option provided" )
            return top

        top.add("&nbsp; matches")
        stmt = 'field_info_list = %s' % field_list_option.replace("\n"," ")
        try:
            exec stmt
        except:
            my.set_configuration_error( top, "badly formed 'field_list' option" )
            return top

        # field_list option should look like this:
        #     [ {'field': 'job_number_prefix', 'size': '1', 'maxlength': '1', 'label': 'Prefix', 'to': 'upper()' },
        #       {'field': 'job_number_year', 'size': '2', 'maxlength': '2', 'label': 'Year (2 digits)', 'to': 'int' },
        #       {'field': 'job_number', 'size': '5', 'maxlength': '5', 'label': 'Number', 'to': 'int' },
        #     ]

        default_size = 10

        for field_info in field_info_list:
            field = field_info.get('field')
            if field:
                text = TextWdg(field)
                size = field_info.get('size')
                if not size:
                    size = default_size

                text.add_attr("size", "%s" % size)
                maxlength = field_info.get('maxlength')
                if maxlength:
                    text.add_attr("maxlength", "%s" % maxlength)

                label = field_info.get('label')
                if label:
                    top.add("&nbsp;&nbsp;&nbsp;%s: " % label)

                top.add(text)

        return top



__all__.append("ButtonFilterElementWdg")
class ButtonFilterElementWdg(BaseFilterElementWdg):

    def alter_search(my, search):

        keys = []
        for key, value in my.values.items():
            if not key.startswith("button"):
                continue
            if value != "on":
                continue

            keys.append(key)

        #if not keys:
        #    search.set_null_filter()


        search.add_op("begin")
        for key in keys:
            button = my.values.get("button_%s" % key)

            option = key.replace("button_", "")

            expression = my.get_option(option)
            if expression:
                sobjects = Search.eval(expression)
                ids = [x.get_id() for x in sobjects]

                search.add_filters("id", ids)

        search.add_op("or")


    def get_display(my):

        from tactic.ui.widget import ActionButtonWdg

        top = my.top

        mode = 'checkbox'

        # FIXME: this is not a great way.  Need to be more specific
        titles = my.get_option("options")
        if titles:
            titles = titles.split("|")
        else:
            titles = []

        table = Table()
        table.add_color("color", "color")
        top.add(table)
        table.add_row()

        if mode == 'button':
            text = HiddenWdg("button")
            top.add(text)
            text.add_class("spt_text")


            for title in titles:
                button = ActionButtonWdg(title=title)
                table.add_cell(button)


                button.add_behavior( {
                'type': 'click_up',
                'title': title,
                'cbjs_action': '''
                var top = bvr.src_el.getParent(".spt_filter_top");
                var text = top.getElement(".spt_text");
                text.value = bvr.title;

                spt.dg_table.search_cbk(evt, bvr);

                setTimeout( function() { text.value = ""; }, 100);

                '''
                } )

        else:

            for i, title in enumerate(titles):
                td = table.add_cell()

                if i != 0:
                    div = DivWdg()
                    div.add_style("float: left")
                    td.add(div)
                    div.add_style("height: 30px")
                    #div.add("&nbsp;")
                    div.add_style("border-style: solid")
                    div.add_style("border-width: 0 1 0 0")
                    div.add_style("margin-right: 15px")
                    div.add_style("border-color: %s" % div.get_color("border"))



                button_title = "button_%s" % title
                checkbox = CheckboxWdg(button_title)
                td.add(checkbox)
                td.add(title)

                if my.values.get(button_title):
                    checkbox.set_checked()
            

        return top






