###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#
__all__ = ["TableActionWdg", "ViewActionWdg"]

from pyasm.common import Xml
from pyasm.web import Widget, DivWdg, HtmlElement, SpanWdg, WebContainer, Table
from pyasm.widget import SelectWdg, FilterSelectWdg, WidgetConfig, HiddenWdg, IconWdg, GeneralAppletWdg


from tactic.ui.common import BaseRefreshWdg


class TableActionWdg(Widget):
    def __init__(my, **kwargs):
        # get the them from cgi
        my.handle_args(kwargs)
        my.target_id = kwargs.get("target_id")

        # FIXME: this is very tenous!!!
        my.table_id = "%s_table" % my.target_id

    #
    # Define a standard format for widgets
    #
    # Get it from web_form_values()
    def get_args_keys(my):
        '''external settings which populate the widget'''
        return {
        'target_id': 'Dom element target to replace the views',
        }

    def handle_args(my, kwargs):
        # verify the args
        args_keys = my.get_args_keys()
        for key in kwargs.keys():
            if not args_keys.has_key(key):
                raise WidgetException("Key [%s] not in accepted arguments" % key)

        web = WebContainer.get_web()
        args_keys = my.get_args_keys()
        for key in args_keys.keys():
            if not kwargs.has_key(key):
                value = web.get_form_value(key)
                kwargs[key] = value




    def get_display(my):

        # add a view action
        view_div = DivWdg()
        view_select = SelectWdg("action|table")
        view_select.add_style("text-align: right")
        view_select.add_empty_option("-- items --")
        view_select.set_option("values", "add|edit|retire|delete|export_all|export_selected")
        view_select.set_option("labels", "Add New Item|X Edit Selected|Retire Selected|Delete Selected|X CSV Export (all)|X CSV Export (selected)")
        view_div.add_style("float: right")
        view_div.add(view_select)

        #view_select.add_event("onchange", "spt.dg_table.retire_selected_cbk('%s')" % my.target_id)
        view_select.add_event("onchange", "spt.dg_table.table_action_cbk(this,'%s')" % my.table_id )

        return view_div


class ViewActionWdg(Widget):
    def __init__(my, **kwargs):
        # get the them from cgi
        my.handle_args(kwargs)
        my.target_id = kwargs.get("target_id")

        # FIXME: this is very tenous!!!
        my.table_id = "%s_table" % my.target_id

    #
    # Define a standard format for widgets
    #
    # Get it from web_form_values()
    def get_args_keys(my):
        '''external settings which populate the widget'''
        return {
        'target_id': 'Dom element target to replace the views',
        }

    def handle_args(my, kwargs):
        # verify the args
        args_keys = my.get_args_keys()
        for key in kwargs.keys():
            if not args_keys.has_key(key):
                raise WidgetException("Key [%s] not in accepted arguments" % key)

        web = WebContainer.get_web()
        args_keys = my.get_args_keys()
        for key in args_keys.keys():
            if not kwargs.has_key(key):
                value = web.get_form_value(key)
                kwargs[key] = value




    def get_display(my):
        # add a view action
        view_div = DivWdg()
        view_select = SelectWdg("action|view_action")
        view_select.add_style("text-align: right")
        view_select.add_empty_option("-- view --")
        view_select.set_option("values", "copy_url|add_my_view|edit|save|rename|delete|custom_property|custom_script")
        view_select.set_option("labels", "X Copy URL to this View|Add to My Views|Edit as Draft|Save Project View As|X Rename View|X Delete View|Add Custom Property|Add Custom Script")
        view_div.add_style("float: right")
        view_div.add(view_select)

        #view_select.add_event("onchange", "spt.dg_table.retire_selected_cbk('%s')" % my.target_id)
        view_select.add_event("onchange", "spt.dg_table.view_action_cbk(this,'%s')" % my.table_id)

        return view_div




"""
# DEPRECATED
class ViewActionOptionWdg(BaseRefreshWdg):
    '''Some views require action options to be present in order to operate
    properly.  This is different from search criteria in that these options
    affect the way an operation may behave.  A good example is the Maya loader.
    There are various modes which are needed to load a maya checkin correctly.
    '''

    def get_args_keys(my):
        '''external settings which populate the widget'''
        return {
        'view': 'view that this widget is making use of',
        'search_type': 'search type',
        'load_script': 'custom load script'

        }

    def init(my):
        my.view = my.kwargs.get('view')
        my.search_type = my.kwargs.get('search_type')
        my.load_script = my.kwargs.get('load_script')
    
    def get_display(my):
        # specially made for "load" view
        if not my.view.endswith("load"):
            return DivWdg()


        widget = Widget()
        # first use
        filter_top = DivWdg(css="maq_search_bar")

        # so dg_table.search_cbk will obtain values from this widget
        filter_top.add_class('spt_table_search')
        filter_top.add_style("background: black")
        filter_top.add_style("padding: 3px")

        # this is used by get_process() in SnapshotLoaderWdg
        filter_top.add(HiddenWdg('prefix', 'view_action_option'))

        for name, value in my.kwargs.items():
            filter_top.set_attr("spt_%s" % name, value)


        from pyasm.common import Container
        from pyasm.prod.web import SnapshotLoaderButtonWdg, IntrospectWdg
        from tactic.ui.cgapp import SObjectLoadWdg

        # this contains the process filter and load options
        sobject_load = SObjectLoadWdg(search_type=my.search_type)
        filter_top.add(sobject_load)

       

        # set the process
        #class foo:
        #    def get_value(my):
        #        return "texture"
        #Container.put("process_filter", foo())

         # add the introspect button
        button = IntrospectWdg()
        filter_top.add( HtmlElement.br() )
        table = Table()
        table.add_class('spt_action_wdg')

        table.set_max_width()
        table.add_row()
        td = table.add_cell( ConnectorSelectWdg() )
        td.add_style('width: 150px')

        app = WebContainer.get_web().get_selected_app()
        span = SpanWdg(app, css='small')
        td = table.add_cell(span)
        td.add_style("padding: 2px")
        icon = IconWdg(icon=eval("IconWdg.%s"%app.upper()), width='13px')
        td.add(icon)
   
        if not Container.get('GeneralAppletWdg'):
            td.add( GeneralAppletWdg() )
            Container.put('GeneralAppletWdg', True)

        td.add(button)

        # create the loader button
        from tactic.ui.cgapp import LoaderElementWdg
        button = SnapshotLoaderButtonWdg()
        if my.load_script:
            loader_script = my.load_script
        else:
            loader_script = SnapshotLoaderElementWdg.get_load_script()
        
        # to be attached
        smart_menu = SnapshotLoaderElementWdg.get_smart_menu()
        button.set_load_script(loader_script)
        button.set_smart_menu(smart_menu)

        td = table.add_cell(button)
        td.add_style('text-align','right')
        td.add_style('padding-right', '40px') 
        widget.add(filter_top)
        widget.add( HtmlElement.br() )
        widget.add(table)

        return widget
"""

"""
## USE ConnectionSelectWdg instead
class ConnectorSelectWdg(Widget):

    def get_display(my):

        widget = DivWdg(id='AppSelectWdg', css='spt_panel')
        widget.add_style('padding-left','6px')
        widget.set_attr('spt_class_name', 'tactic.ui.app.AppSelectWdg') 
        #widget.add_style("float: right")
       

        from tactic.ui.activator import ButtonForDropdownMenuWdg

        menu_data = []
        menu_id = "app_select_menu"
        apps = ['Maya','XSI','Houdini']
        
        for app in apps:
            
            menu_data.append( {
                "type": "action",
                "label": app,
                "bvr_cb": {
                
                    'cbjs_action': "spt.api.Utility.save_widget_setting('app|select','%s'); var activator =spt.ctx_menu.get_activator(bvr);  var top = get_parent_panel(activator); spt.panel.refresh(top, {}, true);" %app 
                }
            } )

        menu = {
            'menu_id': menu_id,
            'width': 140,
            'allow_icons': False,
            'opt_spec_list': menu_data
        }


        button = ButtonForDropdownMenuWdg( id = "%s_Btn" % menu_id,
                                          title = "Select Connector",
                                          menus = [menu],
                                          width =140,
                                          match_w = True)
        #widget.add(app_select)
        widget.add(button)

        return widget
"""

