###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['RepoBrowserWdg', 'RepoBrowserDirListWdg']

from tactic.ui.common import BaseRefreshWdg

from pyasm.common import Environment

from pyasm.web import DivWdg, WebContainer, Table
from pyasm.biz import Snapshot
from pyasm.search import Search, SearchType, SearchKey
from pyasm.widget import IconWdg

from tactic.ui.panel import TableLayoutWdg

from tactic.ui.common import BaseRefreshWdg
from tactic.ui.container import ResizableTableWdg
from tactic.ui.widget import DirListWdg


class RepoBrowserWdg(BaseRefreshWdg):


    def get_files(my):

        search_key = my.kwargs.get("search_key")
        #search_key = "pg/store?project=pg&id=1"
        search_key = "test_project/shoes?project=test_project&code=shoes00001"
        sobject = SearchKey.get_by_search_key(search_key)
        print "sobject: ", sobject

        snapshots = Snapshot.get_by_sobject(sobject)
        print "snapshot: ", snapshots

        paths = []
        for snapshot in snapshots:
            snapshot_paths = snapshot.get_all_lib_paths()
            paths.extend(snapshot_paths)

        #paths = []
        #paths.append("/home/apache/assets/pg/whatever/blue/")
        #paths.append("/home/apache/assets/pg/whatever/a.jpg")
        #paths.append("/home/apache/assets/pg/whatever/")
        #paths.append("/home/apache/assets/scrum/dog/cat/chicken/")
        #paths.append("/home/apache/assets/tickets/foo/foo2/foo3/")


        return paths



    def get_display(my):

        top = my.top
        top.add_color("background", "background")
        top.add_class("spt_repo_browser_top")


        title_wdg = DivWdg()
        top.add(title_wdg)
        title_wdg.add_border()
        title_wdg.add_style("padding: 10px")
        title_wdg.add_gradient("background", "background", -10)
        title_wdg.add("Repository Browser")
        title_wdg.add_style("font-size: 14px")
        title_wdg.add_style("font-weight: bold")


        table = ResizableTableWdg()
        top.add(table)
        table.add_color("color", "color")




        table.add_row()
        left = table.add_cell()

        left.add_border()
        left.add_style("height: 100%")
        left.add_color("background", "background3")


        inner_left = DivWdg()
        left.add(inner_left)
        inner_left.add_style("padding: 10px")
        inner_left.add_style("width: 200px")
        inner_left.add_style("height: 400px")
        inner_left.add_style("overflow-x: hidden")
        inner_left.add_style("overflow-y: auto")
        inner_left.add_class("spt_resizable")


        paths = []
        base_dir = Environment.get_asset_dir()
        dir_list = RepoBrowserDirListWdg(base_dir=base_dir, location="server", paths=paths)
        inner_left.add(dir_list)
        dir_list.add_style("width: 1000")


        right = table.add_cell()
        right.add_border()
        right.add_style("padding: 10px")
        right.add_style("width: 600px")
        right.add_class("spt_repo_browser_content")

        content_div = DivWdg()
        content_div.add_style("min-width: 400px")
        right.add(content_div)


        import os
        paths = my.get_files()
        #base_dir = "%s/" % os.path.dirname(paths[0])

        dir_list = RepoBrowserDirListWdg(base_dir=base_dir, location="server", show_base_dir=True,paths=paths, all_open=True)
        content_div.add(dir_list)


        table.add_row()
        bottom = table.add_cell()
        bottom.add_attr("colspan", "3")
        info_div = DivWdg()
        bottom.add(info_div)
        bottom.add_border()

        info_div.add_style("height: 100px")

        return top


class RepoBrowserDirListWdg(DirListWdg):

    def add_file_behaviors(my, item_div, dirname, basename):
        item_div.add_behavior( {
        'type': 'click_up',
        'dirname': dirname,
        'basename': basename,
        'cbjs_action': '''
        var top = bvr.src_el.getParent(".spt_repo_browser_top");
        var content = top.getElement(".spt_repo_browser_content");
        var class_name = "tactic.ui.tools.repo_browser_wdg.RepoBrowserContentWdg";
        var kwargs = {
          dirname: bvr.dirname,
          basename: bvr.basename
        };
        spt.panel.load(content, class_name, kwargs);
        '''
        } )

    def get_file_icon(my, dir, item):
        return IconWdg.DETAILS

    def get_dir_icon(my, dir, item):
        return IconWdg.LOAD



class RepoBrowserContentWdg(BaseRefreshWdg):

    def get_display(my):

        top = my.top

        dirname = my.kwargs.get("dirname")
        basename = my.kwargs.get("basename")
        path = "%s/%s" % (dirname, basename)

        asset_dir = Environment.get_asset_dir()
        if not dirname.startswith(asset_dir):
            top.add("Error: path [%s] does not belong in the asset directory [%s]" % (path, asset_dir))
            return top

        reldir = dirname.replace(asset_dir, "")
        reldir = reldir.strip("/")

        print "path: ", path
        print "reldir: ", reldir
        print "basenaem: ", basename

        search = Search("sthpw/file")
        search.add_filter("relative_dir", reldir)
        search.add_filter("file_name", basename)
        files = search.get_sobjects()

        top.add(path)
        top.add("<hr/>")

        from tactic.ui.panel import TableLayoutWdg

        print "files: ", files
        for file in files:
            sobject_div = DivWdg()
            top.add(sobject_div)

            snapshot = file.get_parent()
            parent = snapshot.get_parent()
            snapshots = parent.get_related_sobjects("sthpw/snapshot")



            sobject_div.add("<h4>SObject</h4>")
            #search_type = parent.get_base_search_type()
            search_type = parent.get_search_type()
            layout = TableLayoutWdg(search_type=search_type, view='table', mode='simple')
            sobject_div.add(layout)
            layout.set_sobjects( [parent] )


            sobject_div.add("<h4>Snapshot</h4>")
            layout = TableLayoutWdg(search_type='sthpw/snapshot', view='table', mode='simple')
            sobject_div.add(layout)
            layout.set_sobjects( [snapshot] )


            sobject_div.add("<h4>File</h4>")
            layout = TableLayoutWdg(search_type='sthpw/file', view='table', mode='simple')
            sobject_div.add(layout)
            layout.set_sobjects( [file] )

            sobject_div.add("<h4>History</h4>")
            layout = TableLayoutWdg(search_type='sthpw/snapshot', view='table', mode='simple')
            sobject_div.add(layout)
            layout.set_sobjects( snapshots )



        return top





