###########################################################
#
# Copyright (c) 2010, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['DiscussionElementWdg', 'DiscussionWdg', 'DiscussionAddNoteWdg', 'DiscussionAddNoteCmd', 'DiscussionEditWdg', 'NoteStatusEditWdg']

from tactic.ui.common import BaseRefreshWdg, BaseTableElementWdg

from pyasm.common import Environment, TacticException
from pyasm.biz import Pipeline, Project, File, IconCreator
from pyasm.command import Command, EmailTrigger2
from pyasm.web import DivWdg, Table, WikiUtil, HtmlElement, SpanWdg, Widget
from pyasm.search import Search, SearchType, SObject, SearchKey
from pyasm.widget import SwapDisplayWdg, CheckboxWdg, IconButtonWdg, IconWdg, TextWdg, TextAreaWdg, SelectWdg, ProdIconButtonWdg, HiddenWdg
from pyasm.prod.biz import ProdSetting
#from tactic.ui.panel import TableLayoutWdg
from tactic.ui.container import DialogWdg, MenuWdg, MenuItem

import dateutil, os

from tactic.ui.widget.button_new_wdg import ActionButtonWdg, IconButtonWdg

class DiscussionElementWdg(BaseTableElementWdg):

    ARGS_KEYS = {

    'process': {
         'description': 'Explicitly set the process for notes that can be added.',
         'category': 'Options',
         'order': 0
     },
    
    'context': {
        'description': 'Explicitly set the context(s) for notes that can be added.  This can be a list separated by commas',
        'category': 'Options',
        'order': 1
    },
    'use_parent': {
        'description': 'Determines whether or not to enter notes for parent sObject',
        'category': 'Options',
        'type': 'SelectWdg',
        'values': 'true|false',
        'order': 2
    }
    ,
    'show_expand': {
        'description': 'Determines whether or not to show the expand notes icon button',
        'category': 'Options',
        'type': 'SelectWdg',
        'values': 'true|false',
        'order': 3
    },
    'note_format': {
        'description': 'Determines if the notes are shown in compant or full mode',
        'category': 'Options',
        'type': 'SelectWdg',
        'values': 'compact|full',
        'order': 4
    },

    'show_note_status': {
        'description': 'Determines whether or not to show the note status in abbreviated form.',
        'category': 'Options',
        'type': 'SelectWdg',
        'values': 'true|false',
        'order': 5
    },
    # FIXME: need a better name for this
    'context_hidden': {
        'description': 'Determines if the notes in the context are hidden',
        'category': 'Options',
        'type': 'SelectWdg',
        'values': 'true|false',
        'order': 6
    },

    }

    """
    'show_note_expand': {
        'description': 'Determines whether or not to show the entire note or to have to expand to see the full note.',
        'category': 'Options',
        'type': 'SelectWdg',
        'values': 'true|false',
        'order': 4
    },
    """
   
    def is_editable(cls):
        return False
    is_editable = classmethod(is_editable)

    def handle_th(my, th, wdg_idx=None):
        edit_wdg = DiscussionEditWdg()
        my.menu  = edit_wdg.get_menu()

        th.add(edit_wdg)
        
       
   

    def handle_layout_behaviors(my, layout):
        version = my.parent_wdg.get_layout_version()
        if version == "2":
            pass
        else:
            layout.add_smart_styles("spt_discussion_element_top", {
                'margin': '-3px -4px -3px -6px',
                'min-width': '300px'
            } )

        # extra js_action on mouseover to assign the search key of the note to hidden input
        js_action ='''
           var sk_input = menu_top.getElement('.spt_note_action_sk');
           var note_top = bvr.src_el.getParent('.spt_note');
           sk_input.value = note_top.getAttribute('note_search_key');
            '''
        my.menu.set_activator_over(layout, 'spt_note_header', js_action=js_action)
        my.menu.set_activator_out(layout, 'spt_discussion_top')



        layout.add_relay_behavior( {
            'type': 'mouseup',
            'bvr_match_class': 'spt_note_attachment',
            'cbjs_action': '''
            var code_str = bvr.src_el.getAttribute("spt_note_attachment_codes");
            var server = TacticServerStub.get();
            var codes = code_str.split("|");
            for (var i = 0; i < codes.length; i++) {
                // get the files for this snapshot
                var path = server.get_path_from_snapshot(codes[i], {mode:'web'});
                window.open(path);
            }
            '''
        } )


  

    def get_required_columns(my):
        '''method to get the require columns for this'''
        return []

 
    def get_display(my):
        top = DivWdg()
        top.add_class("spt_discussion_element_top")

        sobject = my.get_current_sobject()
        discussion = DiscussionWdg(search_key=sobject.get_search_key(), show_border='false', contexts_checked='false', **my.kwargs)

        top.force_default_context_menu()
        top.add(discussion)


        return top

    def handle_td(my, td):
        td.add_style("padding: 0px")



    def get_header_option_wdg(my):
        #div = DivWdg()
        #button = ActionButtonWdg(title="Expand")
        #div.add(button)
        #return div
        return None



class DiscussionEditWdg(BaseRefreshWdg):

    def init(my):
        my.top_class = 'spt_note_edit_panel'
        my.menu = MenuWdg(mode='horizontal', width = 40, height=16, top_class=my.top_class)
        #my.menu = MenuWdg(mode='horizontal', width = 25, height=20, top_class=my.top_class)

    def get_menu(my):
        return my.menu

    def get_display(my):
        ''' A container div housing the MenuWdg'''
        widget = DivWdg()
        widget.add_class(my.top_class)
        widget.add_styles('position: absolute; display: none')

        hidden = HiddenWdg('note_search_key')
        hidden.add_class('spt_note_action_sk')
        widget.add(hidden)

        
        widget.add(my.menu)

        #menu_item = MenuItem('action', label=IconWdg("Edit", IconWdg.EDIT))
        menu_item = MenuItem('action', label='edit')
        script = [] 
        script.append('''
                    var sk = spt.get_cousin(bvr.src_el, '.spt_note_edit_panel','.spt_note_action_sk');
                    if (sk) sk = sk.value;
                   
                    var server = TacticServerStub.get();
                    var note = null;
                    
                    try {
                        note = server.get_by_search_key(sk);
                    }
                    catch(e) {
                            spt.alert(spt.exception.handler(e));
                        }
                    
                    var ok = function(value) {
                        try{
                            var title = 'Saving Note';
                            server.update(sk, {note: value});
                            var menu = spt.table.get_edit_menu(bvr.src_el);
                            spt.discussion.refresh(menu.activator_el);

                        }
                        catch(e) {
                            spt.alert(spt.exception.handler(e));
                        }
                    }
                    spt.prompt('Edit note [ ' + note.context + ' ]:', ok, 
                    {title: 'Edit',
                    text_input_default: note.note, 
                    okText: 'Save'});

                  

                ''')


        item_behavior = {
            'type': 'click_up',
            'cbjs_action': ';'.join(script)
        }
        menu_item.add_behavior(item_behavior)
        my.menu.add(menu_item)

        #menu_item = MenuItem('action', label=IconWdg("Delete", IconWdg.DELETE))
        menu_item = MenuItem('action', label='delete')
        script ='''
                    
                    var sk = spt.get_cousin(bvr.src_el, '.spt_note_edit_panel','.spt_note_action_sk');

                    if (sk) sk = sk.value;
                   
                    var server = TacticServerStub.get();
                    
                    try {
                        var note = server.get_by_search_key(sk);
                        if (!note) {
                            spt.alert('This note ' + sk +  ' does not exist');
                            return;
                        }
                    }
                    catch(e) {
                            spt.alert(spt.exception.handler(e));
                        }
                    var ok = function() {
                        try{
                            var title = 'Deleting Note';
                                server.delete_sobject(sk);
                                var menu = spt.table.get_edit_menu(bvr.src_el);
                                spt.discussion.refresh(menu.activator_el);


                        }
                        catch(e) {
                            spt.alert(spt.exception.handler(e));
                        }
                    }

                    var cancel = function() {};
                    var note_sum = note.note;
                    if (note.note.length > 50 ) 
                        note_sum =  note.note.substring(0,50) + '...';
                    
                    spt.confirm('Delete this note?<br><br><div style="padding: 10px;border: 1px #aaa dotted">' + note_sum + '</div>' , ok, cancel);

             

                    '''

        item_behavior = {
            'type': 'click',
            'cbjs_action': script
        }
        menu_item.add_behavior(item_behavior)
        my.menu.add(menu_item)
       
        menu_item = MenuItem('action', label='status')
        #menu_item = MenuItem('action', label=IconWdg("Status", IconWdg.FILM))
        script = [] 

     
        script.append('''
                    var menu = spt.table.get_edit_menu(bvr.src_el);
                    var sk = spt.get_cousin(bvr.src_el, '.spt_note_edit_panel','.spt_note_action_sk');
                    if (sk) sk = sk.value;
                   
                    var server = TacticServerStub.get();
                    var wdg = '';
                    try {
                        var class_name = 'tactic.ui.widget.NoteStatusEditWdg';
                        var kwargs = {args: {search_key: sk}};
                        var wdg = server.get_widget(class_name, kwargs);
                    }
                    catch(e) {
                            spt.alert(spt.exception.handler(e));
                        }
                    
                    var ok = function(button) {
                    
                        var server = TacticServerStub.get();
                        var status_sel  = button.getParent('.content').getElement("[name='note_status']");
                        if (status_sel) {
                            var status = status_sel.value;
                            
                            try{
                                var title = 'Saving Note Status';
                                //server.start({title:title, description: title});

                                server.update(sk, {status: status});
                                //server.finish();
                                spt.discussion.refresh(menu.activator_el);
                            }
                            catch(e) {
                                spt.alert(spt.exception.handler(e));
                            }
                        }
                        
                            
                    
                    };
                    
                    spt.prompt('Edit note status:', ok, 
                    {title: 'Edit Note Status',
                     okText: 'Save',
                    custom_html: wdg
                    });
                    '''
                    )

        item_behavior = {
            'type': 'click_up',
            'cbjs_action': ';'.join(script)
        }
        menu_item.add_behavior(item_behavior)
        my.menu.add(menu_item)

        return widget

class DiscussionWdg(BaseRefreshWdg):
    '''Special widget to add work hours'''

    HELP = "discussion-wdg"

    def init(my):
        my.process = ''
        my.contexts = []
        my.use_parent = my.kwargs.get('use_parent') 
        # this indicates if grand parent can be found when use_parent is true
        my.grand_parent = None
        my._load_js = False

    def get_onload_js(my):
        return '''
        spt.discussion = {};
        spt.discussion.refresh = function(src_el) {
            var discussion_top = spt.has_class(src_el, 'spt_discussion_top') ? src_el: src_el.getParent(".spt_discussion_top");

            // find out which contexts are open
            var contexts = discussion_top.getElements(".spt_context");
            var contexts_open = [];
            for (var i = 0; i < contexts.length; i++) {
                if (contexts[i].getAttribute("spt_state") == 'open') {
                    var context = contexts[i].getAttribute("spt_context");
                    if (context.strip())
                        contexts_open.push(context);
                }
            }
            spt.panel.refresh(discussion_top, {contexts_open: contexts_open, is_refresh: 'true'});
        }'''

   
           
    def get_notes(my):
        my.parent = my.kwargs.get("sobject")
        if not my.parent:
            search_key = my.kwargs.get("search_key")
            my.parent = Search.get_by_search_key(search_key)
          

        if my.parent.is_insert():
            return []

        if my.use_parent == 'true':
            if my.parent.has_value('context'):
                my.contexts = [my.parent.get_value('context')]
            if my.parent.has_value('process'):
                my.process = my.parent.get_value('process')

            my.grand_parent = my.parent.get_parent()
            
            if my.grand_parent:
                my.parent = my.grand_parent
            else:
                return []
        else:
            my.process = my.kwargs.get("process")
            # TODO: this needs to be eval to be a list if it's a comma separated string
            my.contexts = my.kwargs.get("context")
            if my.contexts and isinstance(my.contexts, basestring):
                my.contexts = my.contexts.split(',')
                my.contexts =[x.strip() for x in my.contexts if x.strip()]
                
        search = Search("sthpw/note")
        search.add_order_by("process")
        search.add_order_by("context")
        search.add_order_by("timestamp desc")
        if my.process:
            # changed from context back to process
            search.add_filter("process", my.process)
            #search.add_filter("context", my.process)

        #elif my.contexts:
        if my.contexts:
            search.add_filters("context", my.contexts)

        search.add_parent_filter(my.parent)

        notes = search.get_sobjects()
        # FIXME: need to order by pipeline

        from pyasm.biz import Snapshot
        snapshots = Snapshot.get_by_sobjects(notes)
        my.attachments = {}
        for snapshot in snapshots:
            parent_key = snapshot.get_parent_search_key()
            xx = my.attachments.get(parent_key)
            if not xx:
                xx = []
                my.attachments[parent_key] = xx
            xx.append(snapshot)

        return notes
    
    def load_js(my, ele):
        '''add load bvr to the widget at startup or refresh'''
        ele.add_behavior({'type': 'load',
            'cbjs_action': my.get_onload_js()})
        my._load_js = True

    def get_display(my):
       
        my.is_refresh = my.kwargs.get("is_refresh")

        
        my.show_border = my.kwargs.get("show_border")
        if my.show_border in ['false', False]:
            my.show_border = False
        else:
            my.show_border = True


        my.contexts_checked = my.kwargs.get("contexts_checked")
        if my.contexts_checked in ['false', False]:
            my.contexts_checked = False
        else:
            my.contexts_checked = True


        # explicitly set the contexts
        my.contexts = my.kwargs.get("context")
        if my.contexts:
            my.contexts = my.contexts.split(",")
            # remove any spaces
            my.contexts = [x.strip() for x in my.contexts]


        # show context header
        my.show_context_header = my.kwargs.get("show_context_header")
        if my.show_context_header in ['true', True]:
            my.show_context_header = True
        else:
            my.show_context_header = False

            

        # show the number of notes that will start open.  default is 0
        my.num_start_open = my.kwargs.get("num_start_open")
        if my.num_start_open:
            my.num_start_open = int(my.num_start_open)
        else:
            my.num_start_open = 0

        show_expand = my.kwargs.get("show_expand")
        if show_expand in ['true', True]:
            show_expand = True
        else:
            show_expand = False


        my.note_format = my.kwargs.get("note_format")
        if my.note_format in ['', None]:
            my.show_note_expand = my.kwargs.get("show_note_expand")
            if my.show_note_expand in ['true', True]:
                my.note_format = 'full'
            else:
                my.note_format = 'compact'
        elif not my.note_format:
            my.note_format = 'compact'

        my.show_note_status = my.kwargs.get("show_note_status")
        if my.show_note_status in ['true', True]:
            my.show_note_status = True
            my.note_status_dict = ProdSetting.get_dict_by_key('note_status')
        else:
            my.show_note_status = False

        #my.contexts_open = my.kwargs.get("contexts_open")
        from pyasm.web import WebContainer
        web = WebContainer.get_web()
        my.contexts_open = web.get_form_values("contexts_open")
        if not my.contexts_open:
            my.contexts_open = my.kwargs.get("contexts_open")
        if not my.contexts_open:
            my.contexts_open = []



        if my.is_refresh =='true':
            top = Widget()
        else:
            top = DivWdg()
            if not my._load_js:
                my.load_js(top)


            # add a refresh listener
            """
            behavior = {
                'type': 'listen',
                'event_name': 'refresh_note',
                'cbjs_action': '''
                            var menu = bvr.src_el.getParent('.spt_table').getElement('.spt_note_edit_panel');
                            spt.discussion.refresh(menu.activator_el);
                            '''}
                
            top.add_behavior(behavior)
            """
            top.add_class("spt_discussion_top")
            my.set_as_panel(top)
            top.add_style("min-width: 300px")


            max_height = my.kwargs.get("max_height")
            if max_height:
                top.add_style("overflow: auto")
                top.add_style("max-height: %spx" % max_height)

        notes = my.get_notes()
        if my.use_parent == 'true' and not notes and not my.grand_parent:
            sobj = my.parent
            if sobj:
                st_obj = sobj.get_search_type_obj()
                msg = 'Warning: The parent of this [%s] cannot be found. If this happens for every item, try not to set the display option [use_parent] to true.' % st_obj.get_title()
            else:
                msg = 'Warning: The parent cannot be found for this sObject. If this happens for every item, try not to set the display option [use_parent] to true.'
            top.add(msg)
            top.add_style("padding: 5px")
            return top

        code = my.parent.get_code()


        if notes:
            expand_div = DivWdg()
            if show_expand:
                top.add(expand_div)
            expand_div.add_class("spt_discussion_expand")
            #expand_div.add_style("float: right")
            expand_div.add_style("margin: -2px 3px 2px 3px")
            expand_div.add_style("margin-right: 3px")

            expand_wdg = IconButtonWdg(title="Expand in new tab", icon=IconWdg.ZOOM)
            expand_div.add(expand_wdg)

            expand_wdg.add_behavior( {
            'type': 'click_up',
            'search_key': my.parent.get_search_key(),
            'code': code,
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_discussion_top");
            var float = top.getElement(".spt_float");
            var clone = spt.behavior.clone(top);

            expand = clone.getElement(".spt_discussion_expand");
            spt.hide(expand);

            var title = "Notes: " + bvr.code;
            var element_name = bvr.search_key;

            spt.tab.top = $("main_body").getElement(".spt_tab_top");
            spt.tab.add_new(element_name, title);
            spt.tab.load_node(element_name, clone);
            spt.tab.select(element_name);


            var notes = clone.getElements(".spt_note_content");
            for ( var i = 0; i < notes.length; i++ ) {
                spt.show( notes[i] );
            }

            '''
            } )

        # This only shows up if there are no notes
        else:
            no_notes_div = DivWdg()
            top.add(no_notes_div)
            if my.show_border:
                no_notes_div.add_color("background", "background")
                no_notes_div.add_color("color", "color")
            no_notes_div.add_style("padding", "3px")

            add_wdg = ActionButtonWdg(title="+", title2="-", tip='Add a new note', size='small')
            #add_wdg = IconButtonWdg(title="Add New Note", icon=IconWdg.ADD)
            no_notes_div.add(add_wdg)
            add_wdg.add_style("float: right")
            add_wdg.add_style("margin-top: -7px")
            add_wdg.add_style("margin-right: -2px")

            add_wdg.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_discussion_top");
            var add_note = top.getElement(".spt_discussion_add_note");
            spt.toggle_show_hide(add_note);
            '''
            } )

            no_notes_msg = DivWdg()
            no_notes_msg.add_style("opacity: 0.5")
            no_notes_msg.add_style("min-height: 18px")
            no_notes_msg.add_style("width: 90%")
            no_notes_msg.add_style("text-align: center")
            no_notes_div.add(no_notes_msg)
            no_notes_msg.add("<i>-- No notes found --</i>")
            no_notes_div.add_style("font-size: 10px")
            add_note_wdg = DiscussionAddNoteWdg(parent=my.parent, context=my.contexts, process=my.process, use_parent=my.use_parent)
            no_notes_div.add(add_note_wdg)

            return top

        # calculate the number for each context
        my.context_counts = {}
        for note in notes:
            process = note.get_value("process")
            context = note.get_value("context")
            count = my.context_counts.get(context)
            if count == None:
                count = 1
            else:
                count += 1 
            my.context_counts[context] = count


        if my.show_context_header:
            contexts = set()
            for note in notes:
                context = note.get_value("context")
                contexts.add(context)
            contexts_div = DivWdg()
            contexts_div.add_color("color", "color")
            if my.show_border:
                contexts_div.add_border()
            top.add(contexts_div)
            for context in contexts:
                checkbox = CheckboxWdg("context")
                #if my.contexts_checked:
                checkbox.set_checked()
                checkbox.add_behavior( {
                'type': 'change',
                'context': context,
                'cbjs_action': '''

                var value = bvr.src_el.checked;
                var top = bvr.src_el.getParent(".spt_discussion_top");
                var contexts = top.getElements(".spt_context");
                for (var i = 0; i < contexts.length; i++) {
                    if (contexts[i].getAttribute("spt_context") == bvr.context) {
                        if (value == true) {
                            spt.show(contexts[i]);
                        }
                        else {
                            spt.hide(contexts[i]);
                        }
                    }
                }

                var notes = top.getElements(".spt_note");
                for (var i = 0; i < notes.length; i++) {
                    if (notes[i].getAttribute("spt_context") == bvr.context) {
                        if (value == true) {
                            spt.show(notes[i]);
                        }
                        else {
                            spt.hide(notes[i]);
                        }

                    }
                }
 

                '''
                } )
                contexts_div.add(checkbox)
                contexts_div.add(context)
                contexts_div.add("&nbsp;"*3)
            

        # FIXME: need a better name for this
        # determines if each note in the context is hidden
        context_hidden = my.kwargs.get("context_hidden")
        if context_hidden in [True, "true"]:
            context_hidden = True
        else:
            context_hidden = False



        # go through every note and display
        last_context = None
        context_top = None
        context_count = 0
        for i, note in enumerate(notes):
            context = note.get_value("context")
            process = note.get_value("process")
            if last_context == None or context != last_context:
                # organize by context
                context_top = DivWdg()
                context_top.add_class("spt_discussion_context_top")
                context_top.add_class("spt_context")
                #context_top.add_class("hand")
                context_top.add_attr("spt_context", context.encode('utf-8'))
                top.add(context_top)

                if context_hidden or context in my.contexts_open:
                    context_top.add_attr("spt_state", 'open')
                else:
                    context_top.add_attr("spt_state", 'closed')

                context_wdg = my.get_context_wdg(process, context, context_hidden=context_hidden)
                last_context = context
                context_top.add(context_wdg)
                context_top.add_style("min-width: 400px")
                # prefer my.process. if not set, use context 
                #process = my.process

                # NOTE: this is fine if just adding shot level notes, but when dealing with task based notes, it
                # may not be appropriate
                #if not process:

                #    process = context
                
                #add_note_wdg = DiscussionAddNoteWdg(parent=my.parent,context=my.contexts, process=process, use_parent=my.use_parent)
                if my.contexts:
                    context_choices = my.contexts
                else:
                    context_choices = [context]

                add_note_wdg = DiscussionAddNoteWdg(parent=my.parent,context=context_choices, process=my.process, use_parent=my.use_parent)
                context_top.add(add_note_wdg)

                context_count = 0

                #if not my.contexts_checked or not my.show_context_header:
                #    context_top.add_style('display: none')

               


            if my.num_start_open == -1:
                note_hidden = True
            elif context_count >= my.num_start_open:
                note_hidden = True
            else:
                note_hidden = False


            note_wdg = my.get_note_wdg(note, context_hidden=context_hidden, note_hidden=note_hidden)
            if i % 2 == 0:
                note_wdg.add_color("background", "background")
            else:
                note_wdg.add_color("background", "background", -5)
            #context_top.add(note_wdg)
            top.add(note_wdg)

            context_count += 1

        return top

    def get_context_wdg(my, process, context, context_hidden=True):
        div = DivWdg()
        div.add_class("hand")
        
        swap = SwapDisplayWdg()
        div.add(swap)
        swap.add_style("float: left")
        swap.add_style("margin-top: -2")
        swap.add_style("margin-left: -5")

        if context_hidden or context in my.contexts_open:
            swap.set_off()

        # this part doesn't need swap script 
        js_action = '''
                var context = '%s';
                var top = bvr.src_el.getParent(".spt_discussion_top");
                var notes = top.getElements(".spt_note");
                for (var i = 0; i < notes.length; i++) {
                    if (context != notes[i].getAttribute("spt_context") ) {
                        continue;
                    }
                    spt.toggle_show_hide(notes[i]);

                }
                
                var context_el = bvr.src_el.getParent(".spt_context");
                var state = context_el.getAttribute("spt_state");
                if (state == 'open')
                    state = 'closed';
                else
                    state = 'open';
                context_el.setAttribute("spt_state", state);
                ''' %context

        bvr = {
                'type': 'click_up',
                'cbjs_action':  '''
                %s
                %s
                '''%(js_action, swap.get_swap_script())
                }

        swap.add_action_script(js_action)
        
        div.add_behavior(bvr)


        add_wdg = ActionButtonWdg(title="+", title2="-", tip='Add a new note', size='small')
        div.add(add_wdg)
        add_wdg.add_style("float: right")
        add_wdg.add_style("margin-top: -7px")
        add_wdg.add_style("margin-right: -3px")
        #dialog.set_as_activator(add_wdg)
        add_wdg.add_behavior( {
        'type': 'click_up',
        #'dialog_id': dialog_id,
        'process': process,
        'context': context,
        'cbjs_action': '''
        var top = bvr.src_el.getParent(".spt_discussion_context_top");
        var add_note = top.getElement(".spt_discussion_add_note");
        spt.toggle_show_hide(add_note);
        var select = add_note.getElement(".spt_add_note_process");
        
        if (select) {
            var match = bvr.process ? bvr.process : bvr.context;
            for (var i = 0; i < select.options.length; i++) {
                if(select.options[i].value == match)
                    select.selectedIndex = i;
            }
        }
        '''
        } )

        div.add_color("color", "color")
        div.add_style("padding", "4px")
        div.add_gradient("background", "background", -5, -5)
        div.add_style("height", "15px")
        div.add_style("font-weight", "bold")
        div.add_style("margin-bottom", "-1px")
        if not context:
            context = '<i>(no context)</i>'
        div.add(context)

        count_div = SpanWdg()
        div.add(count_div)
        count = my.context_counts.get(context)
        count_div.add(" (%s)" % count)
        count_div.add_style("font-weight: normal")
        count_div.add_style("font-size: 11")
        count_div.add_style("font-style: italic")


        return div




    def get_note_wdg(my, note, context_hidden=True, note_hidden=False):
        context = note.get_value("context")

        div = DivWdg()
        div.add_class("spt_note")
        div.add_attr('note_search_key', note.get_search_key())

        if not context_hidden and context not in my.contexts_open:
            div.add_style("display: none")

        note_value = note.get_value("note") 
        login = note.get_value("login")
        date = note.get_value("timestamp")
        context = note.get_value("context")

        div.add_attr("spt_context", context.encode("UTF-8"))

        content = Table(css='minimal')
        content.add_color("color", "color")

        div.add(content)
        content.add_class("spt_note_top")


      
        content.add_style("width: 100%")

        content.add_row()
        td = content.add_cell()
        #td.add_border()


        title = DivWdg()
        title.add_style("padding: 1px")
        #title.add_color("background", "background", -10)
        title.add_class("spt_note_header")

        title.add("&nbsp;"*3)

        tbody = content.add_tbody()

        if my.note_format == 'full':
            title.add_class("hand")
            swap = SwapDisplayWdg.get_triangle_wdg()
            if note_hidden not in ['true', True]:
                swap.set_off()
            SwapDisplayWdg.create_swap_title(title, swap, tbody)
            title.add(swap)


        # FIXME: something is blocking the checkbox from working
        """
        checkbox = CheckboxWdg("status")
        checkbox.set_attr("checked", "true")
        title.add(checkbox)
        checkbox.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''
        if (bvr.src_el.checked == false) {
            bvr.src_el.checked = "true";
        }
        else {
            bvr.src_el.checked = "false";
        }
        '''
        } )
        """
        #print "date: ", date
        date_obj = dateutil.parser.parse(date)
        display_date_full = date_obj.strftime("%b %d, %Y %H:%M")
        display_date = date_obj.strftime("%b %d - %H:%M")
        title.add(display_date)
        title.add_attr("title", display_date_full)

        if my.note_format == 'full':
            if len(note_value) > 50:
                short_note = "%s ..." % note_value[:48]
            else:
                short_note = note_value
        else:
            # show the entire note
            short_note = WikiUtil().convert(note_value)

        title.add(" - [%s] - %s" % (login, short_note) )

        if my.show_note_status:
            status = note.get_value('status')
            status_label = my.note_status_dict.get(status)
            if status_label:
                title.add (' - %s ' %status_label)

        """
        show_note_edit = 'true'

        if show_note_edit == 'true':
            icon_div = DivWdg(css='spt_note_info')
            icon_div.add_style('display: none')
            icon = IconWdg(icon=IconWdg.INFO)
            icon_div.add(icon)
            # NOT USED cuz it's not that smart: smart menu activator set-up
            #icon_div.add_class("SPT_SMENU_ACTIVATOR")
            #icon_div.set_attr( "SPT_SMENU_SUBSET_TAG_SUFFIX", "NOTE_EDIT_MENU" ) 

         
            icon_div.add_styles('position: absolute; z-index: 100; float: right; top: 0; right: 0; font-size: smaller;')
            title.add_style('position: relative')
            title.add(icon_div)
        """
        td.add(title)

        key = note.get_search_key(use_id=True)
        attachments = my.attachments.get(key)
        if attachments:

            btn = IconButtonWdg(title='View Attachments', icon=IconWdg.ATTACHMENT)
            title.add("&nbsp;");
            btn.add_style("float: right");
            title.add(btn)
            btn.add_class("spt_note_attachment")

            # get the codes to the attachments
            attachment_codes = []
            for attachment in attachments:
                attachment_codes.append( attachment.get_code() )
            btn.add_attr("spt_note_attachment_codes", "|".join(attachment_codes) )


        tbody.add_class("spt_note_content")

        if note_hidden in ['true', True]:
            tbody.add_style("display: none")


        # don't draw the detailed attachment and user info
        if my.note_format == 'compact':
            content.close_tbody()
            return div

        content.add_row()
        left = content.add_cell()
        left.add_style("padding: 10px")
        left.add_style("width: 150px")
        left.add_style("min-height: 100px")
        left.add_style("vertical-align: top")
        left.add_style("border: solid 1px %s" % title.get_color("border"))

        if not login:
            login = "-- No User --"
            left.add(login)
        else:
            from pyasm.security import Login
            from pyasm.widget import ThumbWdg

            login_sobj = Login.get_by_code(login)

            thumb = ThumbWdg()
            thumb.set_icon_size("60")
            if login_sobj:
                thumb.set_sobject(login_sobj)
                left.add(thumb)
                left.add("<br/>")
                left.add_style("font-size: 11px")

                left.add("%s</br>" % login)
                name = "%s %s" % (login_sobj.get_value("first_name"), login_sobj.get_value("last_name") )
                left.add("%s</br>" % name)
                left.add("%s</br>" % login_sobj.get_value("email"))





        right = content.add_cell()
        right.add_style("vertical-align: top")
        right.add_style("padding: 10px")
        right.add_border()

        context = note.get_value("context")

        context_div = DivWdg()
        context_div.add(context)
        context_div.add_style("padding: 5px")
        context_div.add_style("font-size: 14px")
        context_div.add_style("font-weight: bold")
        right.add(context_div)

        right.add( WikiUtil().convert(note_value) )


        attached_div = DivWdg()
        attached_div.add_style("margin-top: 10px")
        snapshots = attachments
        if snapshots:
            attached_div.add("<hr/>Attachments: %s<br/>" % len(snapshots) )

            for snapshot in snapshots:
                thumb_div = DivWdg()
                attached_div.add(thumb_div)
                thumb_div.add_style("float: left")

                thumb = ThumbWdg()
                thumb.set_option('detail','false')
                thumb_div.add(thumb)
                thumb.set_icon_size(60)
                thumb.set_sobject(snapshot)

        right.add(attached_div)

        content.close_tbody()

        return div



class DiscussionAddNoteWdg(BaseRefreshWdg):

        
    def add_style(my, name, value=None):
        my.top.add_style(name, value)



    def get_display(my):

        my.use_parent = my.kwargs.get("use_parent")
        parent = my.kwargs.get("parent")
        if not parent:
            search_key = my.kwargs.get("search_key")
            parent = Search.get_by_search_key(search_key)


        # explicitly set the contexts
        my.contexts = my.get_option("context")

        # need the process to predict the notification to and cc
        my.process = my.kwargs.get('process')
        my.append_contexts = my.get_option("append_context")




        content_div = my.top

        my.set_as_panel(content_div)
        content_div.add_class("spt_discussion_add_note")

        display = my.kwargs.get("display")
        if not display:
            display = "none"
        content_div.add_style("display: %s" % display)

        content_div.add_color("background", "background")
        content_div.add_color("color", "color")
        content_div.add_style("padding: 10px")
        content_div.add_border(style="solid")
        content_div.add_class("spt_add_note_top")

        search_key_hidden = HiddenWdg("search_key")
        search_key_hidden.set_value(parent.get_search_key())
        content_div.add(search_key_hidden)

        #content_div.add_style("height: 250")

        from tactic.ui.app import HelpButtonWdg
        help_button = HelpButtonWdg(alias="notes-widget")
        content_div.add(help_button)
        help_button.add_style("float: right")
        help_button.add_style("margin-top: -5px")



        #if my.contexts:
        #    process_names = my.contexts
        
        if my.process:
            process_names = [my.process]
        else:
            pipeline_code = parent.get_value("pipeline_code", no_exception=True)
            if pipeline_code:
                pipeline = Pipeline.get_by_code(pipeline_code)
                if pipeline:
                    process_names = pipeline.get_process_names()
                else:
                    process_names = ["publish"] 
            else:
                process_names = ["publish"]

        if process_names == ["publish"]:
            hidden = HiddenWdg("process")
            hidden.set_value("publish")
            content_div.add(hidden)
        else:
            wdg_label = "For Process:"
            span = SpanWdg(wdg_label)
            span.add_style('padding-right: 4px')
            content_div.add(span)

            process_select = SelectWdg("process")
            process_select.add_class("spt_add_note_process")
            process_select.set_option("values", process_names)
            content_div.add(process_select)


        # add the context label if it is different from process in use_parent mode
        if my.use_parent and my.contexts:
            hidden =HiddenWdg("context")
            hidden.set_value(my.contexts[0])
            content_div.add(hidden)
            if my.contexts[0] != my.process:
                context_span = SpanWdg(my.contexts[0], css='small')
                content_div.add(context_span)

        content_div.add("<br/>")


        content_div.add("<br>Note:<br/>")
        text = TextAreaWdg("note")
        text.add_style("width: 100%")
        content_div.add(text)

        content_div.add("<br/>"*2)


        #add_button = ProdIconButtonWdg("Submit Note")
        add_button = ActionButtonWdg(title="Submit", tip='Submit information to create a new note')
        content_div.add(add_button)
        add_button.add_style("float: right")

        #ticket_key = Environment.get_ticket()
        add_button.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''

        var note_top = bvr.src_el.getParent(".spt_add_note_top");
        var values = spt.api.get_input_values(note_top, null, false);
        if (values.note == '') {
            spt.alert("Please enter a note before saving");
            return;
        }

        var top = bvr.src_el.getParent(".spt_discussion_add_note");
        var attach_top = top.getElement(".spt_attachment_top");
        var files = attach_top.files;
        var server = TacticServerStub.get();
        var ticket_key = server.start({title: 'New Note'})
        if (typeof(files) != 'undefined') {
            for (var i = 0; i < files.length; i++) {
                spt.app_busy.show("Uploading ...", files[i]);
                server.upload_file(files[i], ticket_key);
            }

            spt.app_busy.hide()

            values['files'] = files;
            values['ticket'] = ticket_key;
        }
        else {
            values['files'] = [];
        }
        var cmd = 'tactic.ui.widget.DiscussionAddNoteCmd';
        
        try{
            server.execute_cmd(cmd, values);
            server.finish();
        }
        catch (e) {
            spt.alert(spt.exception.handler(e));
            server.abort();
        }


    
        spt.discussion.refresh(top);
        '''
        } )





        # attachments
        attachment_div = DivWdg()
        content_div.add(attachment_div)
        attachment_div.add_class("spt_attachment_top")

        #browse_button = ProdIconButtonWdg("Attach")
        browse_button = ActionButtonWdg(title="Attach File(s)", tip='Browse for files to attach to this note')
        attachment_div.add(browse_button)
        browse_button.add_style("float: left")
        browse_button.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''
        spt.app_busy.show("Opening File Browser", "");
        var applet = spt.Applet.get();
        var files = applet.open_file_browser();
        var html = '';
        for (var i = 0; i < files.length; i++) {
            html += files[i] + '<br/>';
        }
        var top = bvr.src_el.getParent(".spt_attachment_top")
        var list = top.getElement(".spt_attachment_list");

        list.innerHTML = html;
        top.files = files;

        spt.app_busy.hide();
        '''
        } )

        attach_list = DivWdg()
        attach_list.add_style('margin-top: 8px')
        attachment_div.add(HtmlElement.br(2))
        attachment_div.add(attach_list)
        attach_list.add_class("spt_attachment_list")
        # too verbose man
        #attach_list.add("<i>-- no attachments --</i>")


        attachment_div.add(HtmlElement.br())

        swap = SwapDisplayWdg()
        content_div.add(swap)
        swap.add_style("float: left")
        title = DivWdg("Mail Options")
        title.add_style("margin: 2px 0px 0px -2px")
        content_div.add(title)

        mail_div = DivWdg()
        SwapDisplayWdg.create_swap_title(title, swap, mail_div)

        content_div.add("<br/>")
        content_div.add(mail_div)
        mail_div.add_class("spt_discussion_mail")
        #mail_div.add_style("display: none")
        mail_div.add("Mail will be sent to:<br/>")

        from pyasm.command import EmailHandler

        # there could be multiple.. so this is not always accurate
        search = Search("sthpw/notification")
        search.add_filter("process", my.process)
        search.add_filter("event", "insert|sthpw/note")
        notification = search.get_sobject()
        if notification:
            handler = EmailHandler(notification, None, None, None)
            to = handler.get_mail_users("mail_to")
            cc = handler.get_mail_users("mail_cc")
            to_emails = []
            cc_emails = []
            for x in to:
                if isinstance(x, SObject):
                    to_emails.append(x.get_value('email'))
                else:
                    to_emails.append(x)
            for x in cc:
                if isinstance(x, SObject):
                    cc_emails.append(x.get_value('email'))
                else:
                    cc_emails.append(x)

           
            mail_div.add("To: %s<br/>" % ",".join(to_emails))
            mail_div.add("Cc: %s<br/>" % ",".join(cc_emails))


        mail_div.add(HtmlElement.br())
        # mail cc and bcc
        mail_div.add("Extra list of emails to send mail to (separated by commas):")



        table = Table()
        table.add_color("color", "color")
        mail_div.add(table)

        # CC
        table.add_row()
        table.add_cell("Cc: ")
        text = TextWdg("mail_cc")
        text.add_style("width: 250px")
        table.add_cell(text)

        # BCC 
        table.add_row()
        table.add_cell("Bcc: ")
        text = TextWdg("mail_bcc")
        text.add_style("width: 250px")
        table.add_cell(text)

        return content_div




class DiscussionAddNoteCmd(Command):

    def get_title(my):
        return "Added a note"

    def execute(my):
        search_key = my.kwargs.get("search_key")
        sobject = Search.get_by_search_key(search_key)

        ticket = my.kwargs.get('ticket')
        note = my.kwargs.get("note")
        mail_cc = my.kwargs.get("mail_cc")
        mail_bcc = my.kwargs.get("mail_bcc")
        if mail_cc:
            mail_cc = mail_cc.split(',')
        else:
            mail_cc = []
        if mail_bcc:
            mail_bcc = mail_bcc.split(',')
        else:
            mail_bcc = []
        process = my.kwargs.get("process")
        context = my.kwargs.get("context")
        if not context:
            context = process

        from pyasm.biz import Note
        note = Note.create(sobject, note, context=context, process=process)
        subject = 'Added Note'
        message = 'The following note has been added for [%s]:\n%s '%(sobject.get_code(), note.get_value('note'))
        project_code = Project.get_project_code()
        users = []
        users.extend(mail_cc)
        users.extend(mail_bcc)
        if len(users) > 0:
            EmailTrigger2.add_notification(users, subject, message, project_code)
            EmailTrigger2.send([],[],[], subject, message, cc_emails=mail_cc,bcc_emails=mail_bcc)


        

        from pyasm.checkin import FileCheckin
        files = my.kwargs.get("files")

        upload_dir = Environment.get_upload_dir(ticket)
        for i, path in enumerate(files):

            path = path.replace("\\", "/")
            basename = os.path.basename(path)
            basename = File.get_filesystem_name(basename) 
            new_path = "%s/%s" % (upload_dir, basename)
            context = "publish"

            file_paths = [new_path]
            source_paths = [new_path]
            file_types = ['main']
            # if this is a file, then try to create an icon
            if os.path.isfile(new_path):
                icon_creator = IconCreator(new_path)
                icon_creator.execute()

                web_path = icon_creator.get_web_path()
                icon_path = icon_creator.get_icon_path()
                if web_path:
                    file_paths = [new_path, web_path, icon_path]
                    file_types = ['main', 'web', 'icon']
                    source_paths.append(web_path)
                    source_paths.append(icon_path)

            checkin = FileCheckin(note, file_paths= file_paths, file_types = file_types, \
                    source_paths=source_paths,  context=context)
            checkin.execute()



class NoteStatusEditWdg(BaseRefreshWdg):
    ''' Custom widget used in the prompt for changing note status'''

    def get_display(my):
        values_map = ProdSetting.get_map_by_key('note_status')
        if not values_map:
            # put in a default
            ProdSetting.create('note_status', 'new:N|read:R|old:O|:', 'map',\
                description='Note Statuses', search_type='sthpw/note')

        labels = []
        values = []
        for key, value in values_map:
            desc = key
            if desc == '':
                desc = '&lt; empty &gt;'
            labels.append('%s'%(desc))
            values.append(key)

        div = DivWdg(css='note_edit_top')
        select = SelectWdg('note_status')
        select.set_option('empty','true')
        select.set_option('values', values)
        select.set_option('labels', labels)
        div.add(select)

        sk = my.kwargs.get('search_key')
        note = SearchKey.get_by_search_key(sk)
        if note:
            current_status = note.get_value('status')
            select.set_value(current_status)

        return div


'''
#Testing of using the gear menu code 
class NoteEditMenuWdg(BaseRefreshWdg):

    def get_display(my):
        from tactic.ui.container import SmartMenu, SmartMenuButtonDropdownWdg
        menus = [ my.get_main_menu(), my.get_status_menu() ]

    
        #from tactic.ui.widget import SingleButtonWdg
        #btn_dd = SingleButtonWdg(title='Global Options', icon=IconWdg.INFO, show_arrow=True)
        btn = IconWdg(icon=IconWdg.INFO)


        #btn_dd.add_behavior( { 'type': 'hover',
        #            'mod_styles': 'background-image: url(/context/icons/common/gear_menu_btn_bkg_hilite.png); ' \
        #                            'background-repeat: no-repeat;' } )
        smenu_set = SmartMenu.add_smart_menu_set( btn, { 'NOTE_EDIT_MENU': menus } )
        btn.add_class( "SPT_SMENU_CONTAINER" )
        #SmartMenu.assign_as_local_activator( btn, "NOTE_EDIT_MENU", True )
        btn_set = SmartMenuButtonDropdownWdg(menus=menus)
        return btn_set


    def get_main_menu(my):
        return { 'menu_tag_suffix': 'MAIN', 'width': 100, 'opt_spec_list': [
            { "type": "action", "label": "Edit",  "bvr_cb": {'cbjs_action': "alert('Edt')"} },
            { "type": "action", "label": "Delete" },
            { "type": "submenu", "label": "Status", "submenu_tag_suffix": "STATUS" },
        ] }


    def get_status_menu(my):
        return {
            'menu_tag_suffix': 'STATUS', 'width': 80, 'opt_spec_list': [

                # { "type": "title", "label": "Edit" },

                { "type": "action", "label": "New",
                    "bvr_cb": {'cbjs_action': "alert(123)"}
                },

                { "type": "action", "label": "Read",
                    "bvr_cb": {'cbjs_action': "alert(123)"}
                },

                { "type": "action", "label": "Old",
                     "bvr_cb": {'cbjs_action':"alert(123)"}
                }

        ] }
'''
